"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/json/stringify"));

var _isInteger = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/number/is-integer"));

var _entries = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/entries"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/includes"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/starts-with"));

var _splice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/splice"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/index-of"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/assign"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/concat"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/keys"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/reduce"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/slice"));

var _context, _context2, _context3, _context4, _context5, _context6, _context7;

const path = require('path');

const glob = require('glob');

const camelcase = require('lodash.camelcase');

const rollupBabel = require('rollup-plugin-babel');

const commonjs = require('rollup-plugin-commonjs');

const nodeResolve = require('rollup-plugin-node-resolve');

const json = require('rollup-plugin-json');

const replace = require('rollup-plugin-replace');

const {
  terser
} = require('rollup-plugin-terser');

const nodeBuiltIns = require('rollup-plugin-node-builtins');

const nodeGlobals = require('rollup-plugin-node-globals');

const {
  sizeSnapshot
} = require('rollup-plugin-size-snapshot');

const omit = require('lodash.omit');

const {
  packageManager
} = require('../jsonate/');

const {
  ifTypescriptProject
} = require('../utils');

const extensions = ['.js', '.jsx', '.es6', '.es', '.mjs', '.ts', '.tsx'];
const {
  getState: getPkgState
} = packageManager();

const {
  useBuiltInBabelConfig,
  parseEnv,
  fromRoot,
  fromConfigs
} = require('../utils');

const pkg = getPkgState().config;

const capitalize = s => s[0].toUpperCase() + (0, _slice.default)(s).call(s, 1);

const minify = parseEnv('BUILD_MINIFY', false);
const format = process.env.BUILD_FORMAT;
const isPreact = parseEnv('BUILD_PREACT', false);
const isNode = parseEnv('BUILD_NODE', false);
const name = process.env.BUILD_NAME || capitalize(camelcase(pkg.name));
const useSizeSnapshot = parseEnv('BUILD_SIZE_SNAPSHOT', false);
const esm = format === 'esm';
const umd = format === 'umd';
const defaultGlobals = (0, _reduce.default)(_context = (0, _keys.default)(pkg.peerDependencies || {})).call(_context, (deps, dep) => {
  deps[dep] = capitalize(camelcase(dep));
  return deps;
}, {});
const deps = (0, _keys.default)(pkg.dependencies || {});
const peerDeps = (0, _keys.default)(pkg.peerDependencies || {});
const defaultExternal = umd ? peerDeps : (0, _concat.default)(deps).call(deps, peerDeps);
const input = glob.sync(fromRoot(process.env.BUILD_INPUT || `src/index.${ifTypescriptProject() ? 'ts' : 'js'}`));
const filenameSuffix = process.env.BUILD_FILENAME_SUFFIX || '';
const filenamePrefix = process.env.BUILD_FILENAME_PREFIX || (isPreact ? 'preact/' : '');
const globals = parseEnv('BUILD_GLOBALS', isPreact ? (0, _assign.default)(defaultGlobals, {
  preact: 'preact'
}) : defaultGlobals);
const external = (0, _filter.default)(_context2 = parseEnv('BUILD_EXTERNAL', isPreact ? (0, _concat.default)(defaultExternal).call(defaultExternal, ['preact', 'prop-types']) : defaultExternal)).call(_context2, (e, i, arry) => (0, _indexOf.default)(arry).call(arry, e) === i);

if (isPreact) {
  delete globals.react;
  delete globals['prop-types'];
  (0, _splice.default)(external).call(external, (0, _indexOf.default)(external).call(external, 'react'), 1);
}

const externalPattern = new RegExp(`^(${external.join('|')})($|/)`);

function externalPredicate(id) {
  const isDep = externalPattern.test(id);
  const isRelative = (0, _startsWith.default)(id).call(id, '.');

  if (umd) {
    // for UMD, we want to bundle all non-peer deps
    return !isRelative && !path.isAbsolute(id) && isDep;
  } // for esm/cjs we want to make all node_modules external
  // TODO: support bundledDependencies if someone needs it ever...


  const isNodeModule = (0, _includes.default)(id).call(id, 'node_modules');
  return isDep || !isRelative && !path.isAbsolute(id) || isNodeModule;
}

const filename = (0, _filter.default)(_context3 = [pkg.name, filenameSuffix, `.${format}`, minify ? '.min' : null, '.js']).call(_context3, Boolean).join('');
const dirpath = path.join(...(0, _filter.default)(_context4 = [filenamePrefix, 'dist']).call(_context4, Boolean));
const output = [{
  name,
  file: path.join(dirpath, filename),
  format: esm ? 'es' : format,
  exports: esm ? 'named' : 'auto',
  globals
}];
const args = (0, _slice.default)(_context5 = process.argv).call(_context5, 2);
const useBuiltinConfig = useBuiltInBabelConfig(args);
const babelPresets = useBuiltinConfig ? [fromConfigs('babelrc.js')] : [];
const replacements = (0, _reduce.default)(_context6 = (0, _entries.default)(umd ? process.env : omit(process.env, ['NODE_ENV']))).call(_context6, (acc, [key, value]) => {
  let val;

  if (value === 'true' || value === 'false' || (0, _isInteger.default)(+value)) {
    val = value;
  } else {
    val = (0, _stringify.default)(value);
  }

  acc[`process.env.${key}`] = val;
  return acc;
}, {});
const config = {
  input: input[0],
  output,
  external: externalPredicate,
  plugins: (0, _filter.default)(_context7 = [isNode ? nodeBuiltIns() : null, isNode ? nodeGlobals() : null, nodeResolve({
    mainFields: ['module', 'main'],
    preferBuiltins: isNode,
    extensions
  }), commonjs({
    include: 'node_modules/**'
  }), json(), rollupBabel({
    exclude: '/**/node_modules/**',
    presets: babelPresets,
    babelrc: !useBuiltinConfig,
    runtimeHelpers: true,
    extensions
  }), replace(replacements), useSizeSnapshot ? sizeSnapshot({
    printInfo: false
  }) : null, minify ? terser() : null]).call(_context7, Boolean)
};
module.exports = config;