"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/keys"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/reduce"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/trim"));

const path = require('path');

const spawn = require('cross-spawn');

const glob = require('glob');

const [executor, ignoredBin, script, ...args] = process.argv;

if (script) {
  spawnScript();
} else {
  var _context, _context2, _context3, _context4;

  const scriptsPath = path.join(__dirname, 'scripts/');
  const scriptsAvailable = glob.sync(path.join(__dirname, 'scripts', '*')); // `glob.sync` returns paths with unix style path separators even on Windows.
  // So we normalize it before attempting to strip out the scripts path.

  const scriptsAvailableMessage = (0, _trim.default)(_context = (0, _filter.default)(_context2 = (0, _map.default)(_context3 = (0, _map.default)(scriptsAvailable).call(scriptsAvailable, path.normalize)).call(_context3, s => s.replace(scriptsPath, '').replace(/__tests__/, '').replace(/\.js$/, ''))).call(_context2, Boolean).join('\n  ')).call(_context);
  const fullMessage = (0, _trim.default)(_context4 = `
Usage: ${ignoredBin} [script] [--flags]

Available Scripts:
  ${scriptsAvailableMessage}

Options:
  All options depend on the script. Docs will be improved eventually, but for most scripts you can assume that the args you pass will be forwarded to the respective tool that's being run under the hood.

May the force be with you.
  `).call(_context4); // eslint-disable-next-line no-console

  console.log(`\n${fullMessage}\n`);
}

function getEnv() {
  var _context5, _context6;

  // this is required to address an issue in cross-spawn
  // https://github.com/kentcdodds/easy-scripts/issues/4
  return (0, _reduce.default)(_context5 = (0, _filter.default)(_context6 = (0, _keys.default)(process.env)).call(_context6, key => process.env[key] !== undefined)).call(_context5, (envCopy, key) => {
    envCopy[key] = process.env[key];
    return envCopy;
  }, {
    [`SCRIPTS_${script.toUpperCase()}`]: true
  });
}

function spawnScript() {
  const relativeScriptPath = path.join(__dirname, './scripts', script);
  const scriptPath = attemptResolve(relativeScriptPath);

  if (!scriptPath) {
    throw new Error(`Unknown script "${script}".`);
  }

  const result = spawn.sync(executor, [scriptPath, ...args], {
    stdio: 'inherit',
    env: getEnv()
  });

  if (result.signal) {
    handleSignal(result);
  } else {
    // eslint-disable-next-line no-process-exit
    process.exit(result.status);
  }
}

function handleSignal(result) {
  if (result.signal === 'SIGKILL') {
    // eslint-disable-next-line no-console
    console.log(`The script "${script}" failed because the process exited too early. ` + 'This probably means the system ran out of memory or someone called ' + '`kill -9` on the process.');
  } else if (result.signal === 'SIGTERM') {
    // eslint-disable-next-line no-console
    console.log(`The script "${script}" failed because the process exited too early. ` + 'Someone might have called `kill` or `killall`, or the system could ' + 'be shutting down.');
  } // eslint-disable-next-line no-process-exit


  process.exit(1);
}

function attemptResolve(...resolveArgs) {
  try {
    return require.resolve(...resolveArgs);
  } catch (error) {
    return null;
  }
}