"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _entries = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/entries"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/reduce"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/assign"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/includes"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/slice"));

const path = require('path');

const spawn = require('cross-spawn');

const glob = require('glob');

const rimraf = require('rimraf');

const yargsParser = require('yargs-parser');

const {
  hasFile,
  resolveBin,
  fromRoot,
  getConcurrentlyArgs,
  writeExtraEntry
} = require('../../utils');

function build() {
  var _context;

  const crossEnv = resolveBin('cross-env');
  const rollup = resolveBin('rollup');
  const args = (0, _slice.default)(_context = process.argv).call(_context, 2);

  const here = p => path.join(__dirname, p);

  const parsedArgs = yargsParser(args);
  const useBuiltinConfig = !(0, _includes.default)(args).call(args, '--config') && !hasFile('rollup.config.js');
  const config = useBuiltinConfig ? `--config ${(p => here(p).replace(process.cwd(), '.'))('../../config/rollup.config.js')}` : (0, _includes.default)(args).call(args, '--config') ? '' : '--config'; // --config will pick up the rollup.config.js file

  const environment = parsedArgs.environment ? `--environment ${parsedArgs.environment}` : '';
  const watch = parsedArgs.watch ? '--watch' : '';
  const sizeSnapshot = parsedArgs['size-snapshot'];
  let formats = ['esm', 'cjs'
  /* , 'umd', 'umd.min' */
  ];

  if (typeof parsedArgs.bundle === 'string') {
    formats = parsedArgs.bundle.split(',');
  }

  const getCommand = (env, ...flags) => {
    var _context2;

    return (0, _filter.default)(_context2 = [crossEnv, 'BUILD_ROLLUP=true', env, rollup, config, environment, watch, ...flags]).call(_context2, Boolean).join(' ');
  };

  const buildPreact = (0, _includes.default)(args).call(args, '--p-react');
  const scripts = buildPreact ? function () {
    const reactCommands = prefixKeys('react.', getCommands());
    const preactCommands = prefixKeys('preact.', getCommands({
      preact: true
    }));
    return getConcurrentlyArgs((0, _assign.default)(reactCommands, preactCommands));
  }() : getConcurrentlyArgs(getCommands());
  const cleanBuildDirs = !(0, _includes.default)(args).call(args, '--no-clean');

  if (cleanBuildDirs) {
    rimraf.sync(fromRoot('dist'));

    if (buildPreact) {
      rimraf.sync(fromRoot('preact'));
    }
  }

  const result = spawn.sync(resolveBin('concurrently'), scripts, {
    stdio: 'inherit'
  });

  if (result.status === 0 && buildPreact && !(0, _includes.default)(args).call(args, '--no-package-json')) {
    writeExtraEntry('preact', {
      cjs: glob.sync(fromRoot('preact/**/*.cjs.js'))[0],
      esm: glob.sync(fromRoot('preact/**/*.esm.js'))[0]
    }, false);
  }

  function prefixKeys(prefix, object) {
    var _context3;

    return (0, _reduce.default)(_context3 = (0, _entries.default)(object)).call(_context3, (cmds, [key, value]) => {
      cmds[`${prefix}${key}`] = value;
      return cmds;
    }, {});
  }

  function getCommands({
    preact = false
  } = {}) {
    return (0, _reduce.default)(formats).call(formats, (cmds, format) => {
      const [formatName, minify = false] = format.split('.');
      const nodeEnv = minify ? 'production' : 'development';
      const sourceMap = formatName === 'umd' ? '--sourcemap' : '';
      const buildMinify = Boolean(minify);
      cmds[format] = getCommand([`BUILD_FORMAT=${formatName}`, `BUILD_MINIFY=${buildMinify}`, `NODE_ENV=${nodeEnv}`, `BUILD_PREACT=${preact}`, `BUILD_SIZE_SNAPSHOT=${sizeSnapshot}`, `BUILD_NODE=${process.env.BUILD_NODE || false}`, `BUILD_REACT_NATIVE=${process.env.BUILD_REACT_NATIVE || false}`].join(' '), sourceMap);
      return cmds;
    }, {});
  }

  return result;
}

module.exports = {
  build
};