"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _endsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/ends-with"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/includes"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/slice"));

const spawn = require('cross-spawn');

const yargsParser = require('yargs-parser');

const {
  resolveBin,
  fromConfigs,
  useBuiltInEslintConfig,
  useBuiltInEslintIgnore,
  print
} = require('../utils');

function lint() {
  try {
    var _context;

    let args = (0, _slice.default)(_context = process.argv).call(_context, 2);
    const parsedArgs = yargsParser(args);
    const useBuiltinConfig = useBuiltInEslintConfig(args);
    const useSpecifiedExtensions = (0, _includes.default)(args).call(args, '--ext');

    if (!useSpecifiedExtensions) {
      args = [...args, '--ext', ['.js', '.jsx', '.ts', '.tsx'].join(',')];
    }

    const config = useBuiltinConfig ? ['--config', fromConfigs('eslintrc.js')] : [];
    const useBuiltinIgnore = useBuiltInEslintIgnore(args);
    const ignore = useBuiltinIgnore ? ['--ignore-path', fromConfigs('eslintignore')] : [];
    const cache = (0, _includes.default)(args).call(args, '--no-cache') ? [] : ['--cache'];
    const filesGiven = parsedArgs._.length > 0;
    const filesToApply = filesGiven ? [] : ['.'];

    if (filesGiven) {
      // we need to take all the flag-less arguments (the files that should be linted)
      // and filter out the ones that aren't js files. Otherwise json or css files
      // may be passed through
      args = (0, _filter.default)(args).call(args, a => {
        var _context2;

        return !(0, _includes.default)(_context2 = parsedArgs._).call(_context2, a) || (0, _endsWith.default)(a).call(a, '.js') || (0, _endsWith.default)(a).call(a, '.ts') || (0, _endsWith.default)(a).call(a, '.tsx');
      });
    }

    const result = spawn.sync(resolveBin('eslint'), [...config, ...ignore, ...cache, ...args, ...filesToApply], {
      stdio: 'inherit'
    });

    if (result.status > 0) {
      // eslint-disable-next-line no-useless-escape
      print(`Linting FAILED ¯\_(ツ)_/¯`); // eslint-disable-next-line no-process-exit

      process.exit(1);
    } else {
      print(`Linting Successful :)`);
    } // We have to process exit here for lintstaged
    // eslint-disable-next-line no-process-exit


    process.exit(result.status);
  } catch (error) {
    // eslint-disable-next-line no-useless-escape
    print(`Linting FAILED ¯\_(ツ)_/¯`);
    print(error); // eslint-disable-next-line no-process-exit

    process.exit(1);
  }
}

(async () => {
  await lint();
})();