"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/slice"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/trim"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/concat"));

/* eslint-disable no-useless-escape */

/* eslint-disable no-console */
const execFileSync = require('child_process').execFileSync;

const chalk = require('chalk');

const semver = require('semver');

require('console.table');

const {
  packageManager
} = require('../jsonate/');

const {
  getProp,
  hasProp
} = packageManager();

function printHint(str) {
  console.log(`${chalk.bold.blue('Hint: ')} ${str}`);
}

const exec = (command, args, verbose) => {
  var _context;

  // eslint-disable-next-line babel/no-unused-expressions
  verbose && console.log(`> ${(0, _concat.default)(_context = [command]).call(_context, args).join(' ')}\n`);
  const options = {
    cwd: process.cwd(),
    env: process.env,
    stdio: 'pipe',
    encoding: 'utf-8'
  };
  const result = execFileSync(command, args, options);
  return result;
};

const execGitCmd = args => {
  const a = exec('git', args);
  return (0, _trim.default)(a).call(a).toString().split('\n');
};

async function getLeaderBoard({
  days = 60
} = {}) {
  const authors = await execGitCmd(['shortlog', '-s', '-n', '-e', '--all', '--since', `${days}.days`, '--no-merges']);
  return authors;
}

function formatShortLog(authors) {
  // git shortlog return no authors
  if (authors.length === 1 && authors[0] === '') {
    return [];
  } else {
    var _context2, _context3;

    return (0, _slice.default)(_context2 = (0, _filter.default)(_context3 = (0, _map.default)(authors // transform the git shortlog authors returned into a json array of objects
    ).call(authors, author => {
      var _context4;

      const element = (0, _trim.default)(author).call(author).split('\t'); // pluck out the email address
      // eslint-disable-next-line prefer-named-capture-group

      const regex = /\<(.*?)\>/g;
      const email = element[1].match(regex)[0];
      const name = (0, _trim.default)(_context4 = element[1].replace(email, '')).call(_context4);
      return {
        name,
        email
      };
    }) // filter out dev ops and ci git users
    ).call(_context3, author => author.email !== '<mirsajjad.mehdi@originenergy.com.au>' && author.email !== '<jenkins@originenergy.com.au>') // only show the top 3
    ).call(_context2, 0, 3);
  }
}

function print(authors = [], period = 'unknown period') {
  // eslint-disable-next-line babel/no-unused-expressions
  authors.length > 0 && console.log(chalk.bold(`Top contributors in the last ${period}:\n`)); // eslint-disable-next-line babel/no-unused-expressions

  authors.length > 0 && console.table(authors);
}

async function printLeaderBoard() {
  const last7Days = formatShortLog((await getLeaderBoard({
    days: 7
  })));
  const last30Days = formatShortLog((await getLeaderBoard({
    days: 30
  })));
  const last6Months = formatShortLog((await getLeaderBoard({
    days: 182
  })));
  console.log(chalk.blue.bold('Questions? Try asking…\n\n'));
  print(last7Days, 'week');
  print(last30Days, 'month');
  print(last6Months, '6 months');
  return {
    last7Days,
    last30Days,
    last6Months
  };
}

exports.printLeaderBoard = printLeaderBoard;

(async () => {
  if (hasProp('engines')) {
    const engines = getProp('engines');
    const version = engines.node;

    if (!semver.satisfies(process.version, version)) {
      throw new Error(`The current node version${process.version} does not satisfy the required version ${version} .`);
    }
  }

  console.log('');
  console.log('');
  await printLeaderBoard();
  printHint(`The README.MD is very helpful. Happy hacking :)`);
  console.log('');
  console.log('');
})();